# Made by Marcos Boggia

import cv2
from spot import Spot
import numpy as np
from pyautogui import screenshot


TM_SQDIFF_NORMED = cv2.TM_SQDIFF_NORMED
TM_CCOEFF_NORMED = cv2.TM_CCOEFF_NORMED
TM_CCORR_NORMED = cv2.TM_CCORR_NORMED


class Detection:

    def __init__(self, tpl, img=None):
        self.tpl = tpl
        if img is None:
            self.img = np.array(screenshot())[:, :, ::-1].copy()
        else:
            self.img = img

    # OpenCV Template Matching

    def tm(self, method, thresh):

        # Create a copy of the images to be used
        img = self.img.copy()
        tpl = self.tpl.copy()

        if thresh:
            _, img = cv2.threshold(img, 127, 255, cv2.THRESH_BINARY)
            _, tpl = cv2.threshold(tpl, 127, 255, cv2.THRESH_BINARY)

        # Apply template Matching with the method
        res = cv2.matchTemplate(img, tpl, method)

        # Grab the Max and Min values, plus their locations
        min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(res)

        if method in [cv2.TM_SQDIFF, cv2.TM_SQDIFF_NORMED]:
            similarity = abs(min_val - 1)
            position = min_loc
        else:
            similarity = max_val
            position = max_loc

        # Create spot wrapper
        spot = Spot(position, tpl.shape)

        return similarity, spot



# Made by Marcos Boggia


# Wraps all position/coordinates calculations for the found spot.
class Spot:

    def __init__(self, position, tpl_shape):
        self.position = position
        self.tpl_shape = tpl_shape

    # Position calculations

    def upper_left_position(self):
        return self.position

    def upper_right_position(self):
        new_y = self.position[1] + self.tpl_shape[1]
        return self.position[0], new_y

    def bottom_left_position(self):
        new_x = int(self.position[0] + self.tpl_shape[0])
        return new_x, self.position[1]

    def bottom_right_position(self):
        new_x = self.position[0] + self.tpl_shape[0]
        new_y = self.position[1] + self.tpl_shape[1]
        return new_x, new_y

    def center_position(self):
        new_x = int(self.position[0] + self.tpl_shape[0] / 2)
        new_y = int(self.position[1] + self.tpl_shape[1] / 2)
        return new_x, new_y

    # This function helps calculate any coordinate inside the image to be detected.
    # Ej: x, y = custom_position(3, 8, 1, 2)
    #   3/8 of the width
    #  __o_____
    # |        |
    # |        |
    # |  x     o 1/2 of the height
    # |        |
    # |________|
    #
    def custom_position(self, x_multiplier, x_modifier, y_multiplier, y_modifier):
        new_x = int(self.position[0] + self.tpl_shape[0] / x_modifier * x_multiplier)
        new_y = int(self.position[1] + self.tpl_shape[1] / y_modifier * y_multiplier)
        return new_x, new_y



# Made by Marcos Boggia

import image_detection as imgd
import mouse


def detect(tpl, similarity_threshold, method=imgd.TM_SQDIFF_NORMED, thresh=False):
    similarity, spot = imgd.Detection(tpl).tm(method, thresh)
    if similarity > similarity_threshold:
        return similarity, spot
    return False


def detect_and_click(tpl, similarity_threshold, clicks=1, method=imgd.TM_SQDIFF_NORMED, thresh=False):
    res = detect(tpl, similarity_threshold, method, thresh)
    if res:
        (similarity, spot) = res
        mouse.click(*spot.center_position(), clicks)
        return True
    return False


def detect_and_hold(tpl, similarity_threshold, time, method=imgd.TM_SQDIFF_NORMED, thresh=False):
    res = detect(tpl, similarity_threshold, method, thresh)
    if res:
        (similarity, spot) = res
        mouse.hold_click(*spot.center_position(), time)
        return True
    return False


# Ej: detect_and_drag(tpl, 0.5, '3/4', '0/1', '7/8', '4/5')
#   3/4 of the width and 0/1 of the height for START
#  __o___o_  7/8 of the width for END
# |  S     |   S = start
# |   \    |   E = end
# |    \   |   \ = the mouse drag path
# |      E o 4/5 of the height for END
# |________|
def detect_and_drag(tpl, similarity_threshold, start_x_fraction, start_y_fraction, end_x_fraction, end_y_fraction,
                    method=imgd.TM_SQDIFF_NORMED, thresh=False):
    res = detect(tpl, similarity_threshold, method, thresh)
    if res:
        (similarity, spot) = res
        start_x, start_y = spot.custom_position(*_values_from_fraction(start_x_fraction),
                                                *_values_from_fraction(start_y_fraction))
        end_x, end_y = spot.custom_position(*_values_from_fraction(end_x_fraction),
                                            *_values_from_fraction(end_y_fraction))
        mouse.drag_click(start_x, start_y, end_x, end_y)
        return True
    return False


# Used to obtain the numbers from fractions like '3/4' etc.
def _values_from_fraction(fraction):
    return int(fraction[0]), int(fraction[2])
